# cs425-G24-mp1

## Name
CS425 -- Machine Project 1!

## Description
This repository represents our efforts to utilize a distributed system to grep log files for Covfefe! Inc, who we are contracted to! The code enclosed in this repository can create a client and server socket setup on multiple machines, which can all query and read from each other. Covfefe! Inc will eventually use our work for their twitter based application designed to fix people's tweets. 

Enclosed in this repository is a `client.py` file and a `server.py` file, which are used to control the client and server respectively. 

## Background Setup
To utilize our code, you will need to install `python3`, `pip`, `faker`, and `git` on all the machines wherein the code will be running. Once all appropriate packages are installed, please clone the repository to each and every pertinent machine (git clone). To do this, you might need to follow basic git setup and create an SSH token.

If you have `python3` set up, you can run the following commands for `pip` and `faker` setups:
- `python3 -m ensurepip --upgrade`
- `pip3 install faker`

## Server Set-up
Once you have all cloned all the repositories and finished background set up, you will need to log into each machine one-by-one (including SSH/RDP if necessary), and move into the respository folder, which is labeled defaultly as "cs425-24-mp1." Once you are here, open up a terminal and run `python3 server.py` to start a server (default port 9999). On any machine where you intend to run a client, you can run `python3 server.py &`, which allows you to continue typing in the same terminal. Otherwise, you can open another terminal on the same machine for the client. The client is where any/all queries are requested from any machine, using the server. 

## Client Set-up
Once you have an opened terminal on a machine wherein you hope to run the client, navigate into the folder representing this repository (default label "cs425-g24-mp1"). Here, if your machine IPs/Ports are different than those given to us by the assignment, run nano "client.py" and edit the IPs and Ports in the machines array located on line 287. After that, return to the directory and run the query `python3 client.py [any flags, queries]`. Replace [any flags, queries] with any of the following:

`--populate True` :
    - This flag will create a randomized log file (with randomized timestamp, level, and message) called "machine.(index).log, where (index) is replaced by index of the machine. This happens on each and every machine will a live server. 

`--test "#"` :
    - This flag will run a certain "#" test on all live machines. The different tests are specified below. Replace the '#' with the corresponding test number. 

`--grep`
    - in the case you want to run a query outside the test cases, replace the test flag with the grep flag, followed by the query you would like to run enclosed in quotes. For example, '--grep "grep agilan"'

Feel free to use the above flags to query all live machines with the client! However, keep in mind that the `--grep` flag cannot be used together with the other two. The `--populate` and `--test` flags can be used together, as when the former is set to `True`, all machines will regenerate random log files before running the unit tests (which takes a few minutes as we are generating large log files), but running it with just a `--test` flag will just run the tests on existing log files (all expected to be in the format `machine.i.log` where `i` is the machine number). So, we suggest running populate once if you haven't run it in your session before to ensure the tests will be run on the log files we designed them for, but afterwards it should not be necessary.

Here are some more examples of commands you can run in your terminal:
- `python3 client.py --grep "grep '\[18/Aug/2022:07:[0-5][0-9]:[0-5][0-9]'"`
- `python3 client.py --grep "grep -C 2 'Chrome'"`
- `python3 client.py --grep "grep -E '(GET|POST)'"`
- `python3 client.py --populate True --test 1`


## Test Cases
There are a variety of test cases already included in the client, which you can run on all machines with a live server using the --test flag, followed by a number. 

Test '1': This test uses "grep" to search for a query that should have 15000 matches in each machine specific log file (if --populate has been used).

Test '2': This test uses "grep" to search for a query that should have 1 match in each machine specific log file (if --populate has been used).

Test '3': This test uses "grep" to search for a query that should have 5000 matches in each machine specific log file (if --populate has been used).

Test '4': This test uses "grep" to search for a query that should have 15000 matches in the machine 2 (at index 2) machine specific log file (if --populate has been used).

Test '5': This test uses "grep" to search for a query that should have 10000 matches in each odd-indexed machine specific log file (if --populate has been used).

Test '6': This test uses "grep" to search for a query that should have 1 match in each machine specific log file (if --populate has been used).

Test '7': This test uses "grep" to search for a query that should have 0 matches in each machine specific log file (if --populate has been used).

Test '8': This test runs all previously mentioned tests at once.

All test cases will have a green success and a red failure message!

## Our Algorithm
All VMs have files client.py (client socket code) and server.py (server socket code). Our process begins by initializing server sockets on all VMs. Through start_server() (server.py), the socket uses TCP protocol for communication. On the VM where results are read, query_machine() (client.py) starts a client socket that connects to a target server socket at a specified IP/port. The server socket reads a passed-in query (through handle_query()), and if it mentions the word “create,” a log file (with random timestamp, level, and message) is created using faker and populated. If not, the subprocess module is used to create a subprocess and run the query (grep) locally on the specified machine log file (hardcoded), and results are passed to the client. The client uses receive_data_in_chunks() to process the data in 1MB chunks. The data is decoded in UTF-8. To fulfill grep capabilities, grep_all_machines() handles creating a grep query and running query_machine() and receive_data_in_chunks() for each and every machine IP in a specified list (our 10 VMs).

## Contributors
Matthew Tyrone Tan (mtan41)
Agilan Gunashankar (agilang2)

Feel free to contact us with any questions!
