import socket
import subprocess
import signal
import sys
import time
import os
import random

#Library to make fake logs
from faker import Faker

fake = Faker()

log_levels = ['INFO', 'WARNING', 'ERROR', 'DEBUG']

#Create a fake log line with the above declared variables
def generate_log_entry():
    timestamp = fake.date_time_this_year().isoformat()
    level = random.choice(log_levels)
    message = fake.text(max_nb_chars=200)
    return f"{timestamp} {level} {message}"

#Generating 60 - 80 MB log files for unit tests
def create_log_file(index):
    
    #Hard coding machine log file names
    filename = f"machine.{index}.log"
    
    with open(filename, 'w') as file:
        
        #This line is guaranteed to appear 5000 times on every machine, but it varies based on the machine number
        for i in range(5000) :
            file.write(f"This is a line rpeated 5000 times on Mchine {index}\n")
            
        #This writes a line 10000 times, but the line varies on even and odd machines.
        if int(index) % 2 == 0:
            for i in range(10000):
                file.write(f"This is a line on Mchine {index}\n")
        else:
            for i in range(10000):
                file.write(f"Hrein exists words presiding upon Mchine {index}\n")
        
        #A line that exists once on every machine.        
        file.write(f"Rre pttern line\n")
        
        #A bunch of random logs
        for i in range(500000):
            random_line = generate_log_entry()
            file.write(random_line + "\n")
            
    print(f"Log file '{filename}' created.")
    


def handle_query(connection):
    try:
        # Process the client
        query = connection.recv(1024).decode('utf-8')
        print("Client connected, processing query " + query)
        
        query_array = query.split(" ")
        
        #Handle a create command (sent if --populate flag is enabled) that will create the randomized log files for the unit tests
        if(query_array[0] == "create"):
            create_log_file(query_array[1])
            connection.sendall("machine."+query_array[1]+".log is ready to be queried")
        else:
            # Execute grep command locally
            result = subprocess.run(query, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

            #Send back the results of the command
            connection.sendall(result.stdout)
            connection.sendall(result.stderr)   

    except Exception as e:
        connection.sendall(str(e).encode('utf-8'))
    finally:
        connection.close()

def start_server(port):  
    #Standard Socket Startup
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_socket.bind(('0.0.0.0', port))
    server_socket.listen(5)
    server_socket.setblocking(False)  # Make the server socket non-blocking
    print(f"Server started on port {port}")

    #Close on CTRL + C
    def signal_handler(sig, frame):
        print("\nGracefully shutting down the server...")
        server_socket.close()
        sys.exit(0)

    # Set up signal handling for graceful shutdown
    signal.signal(signal.SIGINT, signal_handler)

    #Run Server
    while True:
        try:
            try:
                client_socket, addr = server_socket.accept()
                handle_query(client_socket)
            except BlockingIOError:
                # No connection available, continue the loop
                time.sleep(0.1)  # Briefly sleep to avoid busy-waiting
        except KeyboardInterrupt:
            # Handle Ctrl+C
            signal_handler(None, None)
        except Exception as e:
            print(f"Error handling client: {e}")

if __name__ == "__main__":
    start_server(9999)
